`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date: 2020/03/28 16:17:40
// Design Name: 
// Module Name: LAB3_5
// Project Name: 
// Target Devices: 
// Tool Versions: 
// Description: 
// 
// Dependencies: 
// 
// Revision:
// Revision 0.01 - File Created
// Additional Comments:
// 
//////////////////////////////////////////////////////////////////////////////////


module LAB3_5(
    output reg [3:0] digit_active,  // common anode
    output reg [7:0] ssd,           // 7 segment display
    input fcrystal,                 // 100M Hz clock
    input rst_n                     // active low reset
    );
    reg [26:0] counter;             // control scrolling speed
    reg [47:0] content;             // pre-stored pattern NTHUEE
    
    // timer to control the scrolling speed of "NTHUEE" displayed
    always @ (posedge fcrystal or negedge rst_n)
    begin
        if (~rst_n)
            counter <= 0;            // reset
        else
            counter <= counter + 1;  // add 1
    end
    
    // a 8-shifting-step ring-counter for pre-stored pattern "NTHUEE"
    always @ (negedge counter[26] or negedge rst_n)
    begin
        if (~rst_n)
            // letter                          N               T                H               U               E                E
            content <= 48'b11010101_11100001_10010001_10000011_01100001_01100001;
        else begin
            content[ 7:0] <= content[47:40];  // shift 8 steps
            content[47:8] <= content[39: 0];  // shift 8 steps
    end end
    
    // a 2-to-4 1-hot decoder for common anode activation
    always @ (counter[20:19])
        case (counter[20:19])                 // take a suitable refresh frequency
            2'b00:   digit_active <= 4'b1110; // 0
            2'b01:   digit_active <= 4'b1101; // 1
            2'b10:   digit_active <= 4'b1011; // 2
            2'b11:   digit_active <= 4'b0111; // 3
            default: digit_active <= 0;
        endcase
    
    // sending the corresponding signal pattern to the SSD at the right timing
    always @ (counter[20:19])
        case (counter[20:19])                 // take a suitable refresh frequency
            2'b00:   ssd <= content[ 7: 0];   //  first digit (letter)
            2'b01:   ssd <= content[15: 8];   // second digit (letter)
            2'b10:   ssd <= content[23:16];   //  third digit (letter)
            2'b11:   ssd <= content[31:24];   //   last digit (letter)
            default: ssd <= 0;
        endcase
    
endmodule                                     // LAB3_5
